#define OPTIMIZE_ROM_CALLS
//#define RETURN_VALUE
#define ENABLE_ERROR_RETURN
#define SAVE_SCREEN

#include <tigcclib.h>
typedef unsigned char byte;

byte grid[9][9];
byte restricted[9][9][9];

byte getBox(byte squ);
byte getCol(byte squ);
byte getGridNumber(byte squ);
byte getRow(byte squ);
void defineRestrictedMatrix();
byte newGrid();
void processAllSquares();
void setSecuredValues();
void removeFromBox(byte box, byte num);
void updateIO();

void puzzle1();

/**
	*Main function for solving a given Sudoku puzzle
*/
void _main(void)
{
	byte r;
	defineRestrictedMatrix();
	if(!newGrid())
		exit(40);
	puzzle1();
	updateIO();
	for(r=0; r<23; r++)
	{
		setSecuredValues();
		processAllSquares();
	}
	updateIO();
}

/**
	*Finalize secured values by putting them into the grid
*/
void setSecuredValues()
{
	byte i,j,k,c=0,val,t;
	for(i=0; i<9; i++)
	{
		for(j=0; j<9; j++)
		{
			for(k=0; k<9; k++)
			{
				t = restricted[i][j][k];
				c += t;
				if(t!=0)
					val = k+1;
			}
			if(c == 1)
			{
				grid[i][j] = val;
			}
			c = 0;
			/*--------------------------------------------------------*/
			for(k=0; k<9; k++)
			{
				t = restricted[i][k][i];
				c += t;
				if(t!=0)
					val = i+1;
			}
			if(c == 1)
			{
				grid[i][j] = val;
			}
			c = 0;
			/*--------------------------------------------------------*/
			for(k=0; k<9; k++)
			{
				t = restricted[k][j][i];
				c += t;
				if(t!=0)
					val = i+1;
			}
			if(c == 1)
			{
				grid[i][j] = val;
			}
			c = 0;
		}
	}
}

/**
	*Remove the possibility of a defined square from every square
	*in its row, column, box, and from its own restrictions.
*/
void processAllSquares()
{
	byte i,j,k,num;
	for(i=0; i<9; i++)
	{
		for(j=0; j<9; j++)
		{
			num = grid[i][j];
			if(num != 0)
			{
				for(k=0; k<9; k++)
				{
					restricted[k][j][num-1] = 0;
					restricted[i][k][num-1] = 0;
					restricted[i][j][k] = 0;
				}
				restricted[i][j][num-1] = 1;
				removeFromBox(getBox((i+1)*(j+1)),num);
			}
		}
	}
}

/**
	*Remove the given possibility from the given box.
	*
	*Input box 1-9; possibility 1-9
*/
void removeFromBox(byte box, byte num)
{
	byte first = 1 + --box/3*27 + 3*(box%3), add[] = {0,1,2,9,10,11,18,19,20}, i=0, e;
	for(; i<9; i++)
	{
		e = first + add[i];
		restricted[getRow(e)-1][getCol(e)-1][num-1] = 0;
	}
}

/**
	*Find which of the nine major boxes a given square lies in.
	*
	*Input square 1-81
	*Output box 1-9
*/
byte getBox(byte squ)
{
	return (getCol(squ)-1)/3 + 1 + (getRow(squ)-1)/3*3;
}

/**
	*Restricts the possible values for the squares
*/
void defineRestrictedMatrix()
{
	byte i,j,k;
	for(i=0; i<9; i++)
		for(j=0; j<9; j++)
			for(k=0; k<9; k++)
				restricted[i][j][k] = 1;
}



/**
	*Initialize a new grid from the 81 arguments (or zero arguments)
	*that are passed into the function from the
	*TI-OS home screen.
*/
byte newGrid()
{
	byte r,c,re = ArgCount();
	if(re>81)
		exit(40);
	re = TRUE;
	ESI q = top_estack;
	for(r=0; r<9; r++)
		for(c=0; c<9; c++)
		{
			grid[r][c] = GetIntArg(q);
			if(grid[r][c]>9)
				re = FALSE;
		}
	return re;
}

/**
	*Clear the screen.
	*Display the grid in a crude but understandable format
*/
void updateIO()
{
	FontSetSys(0);
	byte r, c;
	clrscr();
	for(r=0; r<9; r++)
	{
		for(c=0; c<9; c++)
			{
				if((c+1)%3 == 1)
					printf(" | ");
				if(grid[r][c] != 0)
					printf(" %d ",grid[r][c]);
				else
					printf("    ");
			}
			if((r+1) % 3 == 0)
				printf("|\n-----------------------\n");
			else
				printf("|\n");
	}
	ngetchx();
}


/**
	*Get the column that a given square lies in.
	*
	*Input square 1-81
	*Output column 1-9
*/
byte getCol(byte squ)
{
	byte re = squ % 9;
	return (re==0) ? 9 : re;
}

/**
	*Get the row that a given square lies in.
	*
	*Input square 1-81
	*Output row 1-9
*/
byte getRow(byte squ)
{
	return squ/9 + (squ%9!=0);
}

void puzzle1() //Easy Puzzle 111,532,034
{
	byte puzzle[9][9] =
	 {{0,4,0,3,5,0,0,0,0},
	  {0,0,3,1,0,0,0,8,0},
	  {9,0,1,0,8,0,7,0,3},
	  {0,7,6,0,9,0,0,4,0},
	  {5,0,9,0,4,0,3,0,8},
	  {0,8,0,0,1,0,6,5,0},
	  {1,0,5,0,7,0,2,0,6},
	  {0,6,0,0,0,1,8,0,0},
	  {0,0,0,0,3,5,0,1,0}};
	
	byte i,j;
	for(i=0; i<9; i++)
		for(j=0; j<9; j++)
			grid[i][j] = puzzle[i][j];
}